/********************************************************************
* FileName:		PARLE_test.C, realise a partie de MICROCHIP AN643, version simplifiee pour PIC18F2620
* Dependencies: adpcm.h, mpfs.h, string.h, delays.h
* Processor:	PIC18F2620 8MHz SORTIE SON PWM sur CCPR1
* Lors de l appui sur S3(PB0) le systeme dit le nombre converti sur AN0(potentiometre ) conversion sur 8 bits (0 - 255)
* capacite d'enregistrement dans P18F2620 : envron 16s
*********************************************************************/
#include <p18f2620.h>

#include "adpcm.h"	// decodage ADPCM
#include "mpfs.h"	// gestionnaire fichier MICROCHIP
#include <delays.h>

// equivalences
#define et	0		// mot et
#define unite	1	// mot unite

// cent dix un represente le chiffre centaines dizaines unites de nombre
typedef struct {
				unsigned char cent;
				unsigned char dix;
				unsigned char un;
			   } nombre;

// prototypes
void PlayClip(MPFS hfile);						// Play specified audio clip
void Transistion(void);							// Transition between clips
void cdu(unsigned char x, nombre *nb);			// calcul centaines dizaines unites de x et range dans une structure de tupe nombre
unsigned int mesvolt(void);						// ADC sur AN0
void ditnb(unsigned char n);					// dit (PWM) le nombre n du fichier
void dittxt(unsigned char n);					// dit (PWM) le texte t du fichier

// variables globales
struct  ADPCMstate   state;         			// ADPCM state variable library adpcm


/*****************************************************************************
*   PROGRAMME PRINCIPAL                                 *
*****************************************************************************/
void main(void)
{
unsigned char n; 	// nombre issu de ADC
nombre nbadire;		// structure pour le nombre a dire 

	OSCCON = 0x60;								// Enable 4MHz, no IDLE // PWM calculee pour Q=32MHz (4x8Mhz)
	OSCTUNEbits.PLLEN = 1;						// Enable PLL

	// Setup input/outputs
	TRISBbits.TRISB0 = 1; // PB0 en entree (bouton S3)		
	TRISCbits.TRISC2 = 0; // PRC2 (CCPR1) en sortie
	
	// Analog 0 On
	ADCON0=1;				// CAN on. CLOCK=FOSC/2. CANAL0 (RA)
	ADCON1=0b10001110;	// justification  droite, seul AN0 est activ, VREF+=VDD VREF-=VSS
	ADCON2=0b10000000;

	// Setup PWM 
	T2CON = 0x0C;	
	PR2 = 249;
	CCPR1L = 0;
	CCP1CON = 0x0c;

// commentaires a retirer si interruption
//	RCONbits.IPEN = 1;						// Enable priority interrupts
//	INTCONbits.GIEH = 1;					// Enable global interrupts
	
	MPFSInit();								// Initialize "FAT" for audio clips

   while(1)
   {
		while (PORTBbits.RB0);		// attend PRB0=0
		while (!PORTBbits.RB0);		// attend PRB0=1

			n=(unsigned char)(mesvolt()>>2);	 // mesure VAN0 et tronque sur 8bits
			cdu(n,&nbadire); 	// isole cent dix et un de n et range dans structure nbadire
			if (nbadire.cent) {
									if (nbadire.cent>1)ditnb(nbadire.cent);
									ditnb(100);											// dit les centaines
							  }					
			if ((nbadire.dix>1 && nbadire.dix<7) || (nbadire.dix==8) ) ditnb(nbadire.dix*10);	//20 30 40 50 60 80
			else if ((nbadire.dix==7) || (nbadire.dix==9) ) ditnb((nbadire.dix-1)*10);			// dire 60 pour 70 et 90
			if ((nbadire.dix>1 && nbadire.dix<8 && nbadire.un==1)  ) dittxt(et);			// -et- pour 21 31 41 51 61 71
			if (nbadire.dix==1 || nbadire.dix==7 || nbadire.dix==9) ditnb(10+nbadire.un); 		// 10 11 12 13 14 15 16 17 18 19
			else if (nbadire.un) ditnb(nbadire.un);											// 1 2 3 4 5 6 7 8 9
			if (!nbadire.cent && !nbadire.dix && !nbadire.un ) ditnb(0);					// 0

			dittxt(unite);		// dit simplement -unite-
	}
}

//----------------------------------------------------------------------------
// dit le fichier -nombre d-
//----------------------------------------------------------------------------
void ditnb(unsigned char d)
{
char filename[13];							// filename to play
MPFS hFile;									// file pointer
nombre nmb;					// structure type nombre
unsigned char i;
		i=0;
		cdu(d,&nmb);		// isole cent dix et un de n et range dans structure nmb
		if (nmb.cent) { 	// nombre de type cen, dix,un
						filename[i++] = nmb.cent+0x30;													
						filename[i++] = nmb.dix+0x30;
						filename[i++] = nmb.un+0x30;
					  }		
	    else if (nmb.dix)	// nombre de type dix,un
					  { 
    					filename[i++] = nmb.dix+0x30;
						filename[i++] = nmb.un+0x30;
					  }	
		else filename[i++] = nmb.un+0x30; // nombre avec uniquement les unites
// complete l'extension .dat
		filename[i++] = '.';
		filename[i++] = 'd';
		filename[i++] = 'a';
		filename[i++] = 't';
		filename[i++] = 0;
		hFile = MPFSOpen(filename);				// Open file
		PlayClip(hFile);						// Play audio clip
}

//----------------------------------------------------------------------------
// dit le fichier -texte t-
//----------------------------------------------------------------------------
void dittxt(unsigned char t)
{
char filename[13];							// filename to play
MPFS hFile;									// file pointer
unsigned char i;
i=0;
	switch (t)
	{
		case 0 :	
				filename[i++] = 'e';
				filename[i++] = 't';
				filename[i++] = '.';
				filename[i++] = 'd';
				filename[i++] = 'a';
				filename[i++] = 't';
				filename[i] = 0;
				break;
		case 1 :
				filename[i++] = 'u';
				filename[i++] = 'n';
				filename[i++] = 'i';
				filename[i++] = 't';
				filename[i++] = 'e';
				filename[i++] = '.';
				filename[i++] = 'd';
				filename[i++] = 'a';
				filename[i++] = 't';
				filename[i] = 0;
				break;
	}
		hFile = MPFSOpen(filename);				// Open file
		PlayClip(hFile);						// Play audio clip
}
//----------------------------------------------------------------------------
// retourne ADC0
//----------------------------------------------------------------------------
unsigned int mesvolt(void)
{		
		ADCON0bits.GO_DONE=1;		// SOC
		while(ADCON0bits.GO_DONE);	// attend EOC
		return (ADRES);				// calcule la tension
}

//----------------------------------------------------------------------------
// isole centaines dizaines et unite
//----------------------------------------------------------------------------
void cdu(unsigned char x, nombre *nb)
{
unsigned char c,d,u;
c=x/100;			// calcul chiffre centaines
d=(x-c*100)/10;		// calcul chiffre dizaine
u=x-c*100-d*10;		// calcul chiffre unite
nb->cent=c;			// range c dans structure
nb->dix=d;			// range d dans structure
nb->un=u;			// range u dans structure
}

//----------------------------------------------------------------------------
// FONCTIONS DE PLAYBACK
//----------------------------------------------------------------------------
void Transistion(void)
{
	if(CCPR1L > 64)									// If duty cycle is greater than midpoint
	{
		Delay100TCYx(20);							// delay
		CCPR1L--;	
	}
	else											// duty cycle less than midpoint
	{
		Delay100TCYx(20);							// delay
		CCPR1L++;									// increment duty cycle
	}
	Delay100TCYx(20);								// final delay
	return;
}

void PlayClip(MPFS hfile)
{
	unsigned char code;
	unsigned short sample;
	
	Transistion();									// transistion PWM output to mid level
	
	state.prevsample=32768;                 		// configure initial state for ADPCM
	state.previndex=0;                  			//  same

	if(hfile != MPFS_INVALID)						// if file is valid
	{
		if(MPFSGetBegin(hfile))						// data in file
		{
			while(!MPFSIsEOF())						// Is character end of file
			{
				code = MPFSGet();					// read code from memory
				PIR1bits.TMR2IF = 0;				// clear Timer2 interrupt flag
				// Send the upper 4-bits of code to decoder
				sample = ADPCMDecoder((code>>4)&0x0f, &state);
				while(!PIR1bits.TMR2IF);			// Wait for Timer2 interrupt flag (8KHz)
				CCPR1L = (unsigned char)(sample>>8);// Load PWM duty cycle with 8-bits

				PIR1bits.TMR2IF = 0;				// clear Timer2 interrupt flag
				// Send the lower 4-bits of code to decoder	
				sample = ADPCMDecoder(code&0x0f,&state);
				while(!PIR1bits.TMR2IF);			//Wait for Timer2 interrupt flag (8KHz)
				CCPR1L = (unsigned char)(sample>>8);// Load PWM duty cycle with 8-bits
			}
			hfile = MPFSGetEnd();					// End access to file
		}
	}
	MPFSClose();									// Close file
	Delay10KTCYx(20);
	return;
}

// ------------------------------ FIN FICHIER ---------------------------------------------

	
